@@ -1,12 +1,12 @@
 import hashlib
 from collections import defaultdict
 from dataclasses import MISSING, Field as _Field
-from datetime import datetime, date, time, tzinfo
+from datetime import datetime, date, time, tzinfo, timezone
 from typing import TYPE_CHECKING, Any, TypedDict, cast
 from zoneinfo import ZoneInfo
 
 from .decorators import setup_recursive_safe_function
-from ..constants import PY310_OR_ABOVE, PY311_OR_ABOVE
+from ..constants import PY310_OR_ABOVE, PY311_OR_ABOVE, PY314_OR_ABOVE
 from ..log import LOG
 from ..type_def import DefFactory, ExplicitNull, PyNotRequired, NoneType
 from ..utils.function_builder import FunctionBuilder
@@ -20,8 +20,7 @@ if TYPE_CHECKING:  # pragma: no cover
 
 
 # UTC Time Zone
-UTC = ZoneInfo('UTC')
-
+UTC = timezone.utc
 
 _BUILTIN_COLLECTION_TYPES = frozenset({
     list,
@@ -451,6 +450,44 @@ UTCDateTimePattern = PatternBase(datetime, tz_info=UTC)
 UTCTimePattern = PatternBase(time, tz_info=UTC)
 
 
+def _normalize_alias_path_args(all_paths, load, dump):
+    """Normalize `AliasPath` arguments and canonicalize path values."""
+    if load is not None:
+        all_paths = load
+        load = None
+        dump = ExplicitNull
+
+    elif dump is not None:
+        all_paths = dump
+        dump = None
+        load = ExplicitNull
+
+    if isinstance(all_paths, str):
+        all_paths = (split_object_path(all_paths),)
+    else:
+        all_paths = tuple([
+            split_object_path(a) if isinstance(a, str) else a
+            for a in all_paths
+        ])
+
+    return all_paths, load, dump
+
+
+def _normalize_alias_args(default, default_factory, all_aliases, load, dump):
+    """Normalize `Alias` arguments and canonicalize alias values."""
+
+    if default is not MISSING and default_factory is not MISSING:
+        raise ValueError('cannot specify both default and default_factory')
+
+    if all_aliases:
+        load = dump = all_aliases
+
+    elif load is not None and isinstance(load, str):
+        load = (load,)
+
+    return all_aliases, load, dump
+
+
 # Instances of Field are only ever created from within this module,
 # and only from the field() function, although Field instances are
 # exposed externally as (conceptually) read-only objects.
@@ -462,11 +499,125 @@ UTCTimePattern = PatternBase(time, tz_info=UTC)
 # When cls._FIELDS is filled in with a list of Field objects, the name
 # and type fields will have been populated.
 
+# In Python 3.14, dataclasses adds a new parameter to the :class:`Field`
+# constructor: `doc`
+#
+# Ref: https://docs.python.org/3.14/library/dataclasses.html#dataclasses.field
+if PY314_OR_ABOVE:
+    # noinspection PyPep8Naming,PyShadowingBuiltins
+    def Alias(
+        *all,
+        load=None,
+        dump=None,
+        skip=False,
+        default=MISSING,
+        default_factory=MISSING,
+        init=True,
+        repr=True,
+        hash=None,
+        compare=True,
+        metadata=None,
+        kw_only=False,
+        doc=None,
+    ):
+
+        all, load, dump = _normalize_alias_args(default, default_factory, all, load, dump)
+
+        return Field(
+            load,
+            dump,
+            skip,
+            None,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+            kw_only,
+            doc,
+        )
+
+    # noinspection PyPep8Naming,PyShadowingBuiltins
+    def AliasPath(
+        *all,
+        load=None,
+        dump=None,
+        skip=False,
+        default=MISSING,
+        default_factory=MISSING,
+        init=True,
+        repr=True,
+        hash=None,
+        compare=True,
+        metadata=None,
+        kw_only=False,
+        doc=None,
+    ):
+        all, load, dump = _normalize_alias_path_args(all, load, dump)
+
+        return Field(
+            load,
+            dump,
+            skip,
+            all,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+            kw_only,
+            doc,
+        )
+
+    class Field(_Field):
+
+        __slots__ = ("load_alias", "dump_alias", "skip", "path")
+
+        # noinspection PyShadowingBuiltins
+        def __init__(
+            self,
+            load_alias,
+            dump_alias,
+            skip,
+            path,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+            kw_only,
+            doc=None,
+        ):
+
+            super().__init__(
+                default,
+                default_factory,
+                init,
+                repr,
+                hash,
+                compare,
+                metadata,
+                kw_only,
+                doc,
+            )
+
+            self.load_alias = load_alias
+            self.dump_alias = dump_alias
+            self.skip = skip
+            self.path = path
+
+
 # In Python 3.10, dataclasses adds a new parameter to the :class:`Field`
 # constructor: `kw_only`
 #
 # Ref: https://docs.python.org/3.10/library/dataclasses.html#dataclasses.dataclass
-if PY310_OR_ABOVE:  # pragma: no cover
+elif PY310_OR_ABOVE:  # pragma: no cover
 
     # noinspection PyPep8Naming,PyShadowingBuiltins
     def Alias(*all,
@@ -479,17 +630,22 @@ if PY310_OR_ABOVE:  # pragma: no cover
               hash=None, compare=True,
               metadata=None, kw_only=False):
 
-        if default is not MISSING and default_factory is not MISSING:
-            raise ValueError('cannot specify both default and default_factory')
+        all, load, dump = _normalize_alias_args(default, default_factory, all, load, dump)
 
-        if all:
-            load = dump = all
-
-        elif load is not None and isinstance(load, str):
-            load = (load, )
-
-        return Field(load, dump, skip, None, default, default_factory, init, repr,
-                     hash, compare, metadata, kw_only)
+        return Field(
+            load,
+            dump,
+            skip,
+            None,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+            kw_only,
+        )
 
     # noinspection PyPep8Naming,PyShadowingBuiltins
     def AliasPath(*all,
@@ -501,28 +657,22 @@ if PY310_OR_ABOVE:  # pragma: no cover
                   init=True, repr=True,
                   hash=None, compare=True,
                   metadata=None, kw_only=False):
+        all, load, dump = _normalize_alias_path_args(all, load, dump)
 
-        if load is not None:
-            all = load
-            load = None
-            dump = ExplicitNull
-
-        elif dump is not None:
-            all = dump
-            dump = None
-            load = ExplicitNull
-
-        if isinstance(all, str):
-            all = (split_object_path(all), )
-        else:
-            all = tuple([
-                split_object_path(a) if isinstance(a, str) else a
-                for a in all
-            ])
-
-        return Field(load, dump, skip, all, default, default_factory, init, repr,
-                     hash, compare, metadata, kw_only)
-
+        return Field(
+            load,
+            dump,
+            skip,
+            all,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+            kw_only,
+        )
 
     class Field(_Field):
 
@@ -556,18 +706,21 @@ else:  # pragma: no cover
               init=True, repr=True,
               hash=None, compare=True, metadata=None):
 
-        if default is not MISSING and default_factory is not MISSING:
-            raise ValueError('cannot specify both default and default_factory')
+        all, load, dump = _normalize_alias_args(default, default_factory, all, load, dump)
 
-        if all:
-            load = dump = all
-
-        elif load is not None and isinstance(load, str):
-            load = (load, )
-
-        return Field(load, dump, skip, None,
-                     default, default_factory, init, repr,
-                     hash, compare, metadata)
+        return Field(
+            load,
+            dump,
+            skip,
+            None,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+        )
 
     # noinspection PyPep8Naming,PyShadowingBuiltins
     def AliasPath(*all,
@@ -579,28 +732,21 @@ else:  # pragma: no cover
                   init=True, repr=True,
                   hash=None, compare=True,
                   metadata=None):
+        all, load, dump = _normalize_alias_path_args(all, load, dump)
 
-        if load is not None:
-            all = load
-            load = None
-            dump = ExplicitNull
-
-        elif dump is not None:
-            all = dump
-            dump = None
-            load = ExplicitNull
-
-        if isinstance(all, str):
-            all = (split_object_path(all), )
-        else:
-            all = tuple([
-                split_object_path(a) if isinstance(a, str) else a
-                for a in all
-            ])
-
-        return Field(load, dump, skip, all, default, default_factory, init, repr,
-                     hash, compare, metadata)
-
+        return Field(
+            load,
+            dump,
+            skip,
+            all,
+            default,
+            default_factory,
+            init,
+            repr,
+            hash,
+            compare,
+            metadata,
+        )
 
     class Field(_Field):
 
