@@ -1,10 +1,9 @@
-from typing import ClassVar, Generic, Union, List, Tuple, Dict, Callable
+from typing import ClassVar, Generic, Union, List, Tuple, Dict, Callable, Literal
 
 import pytest
 
 from dataclass_wizard.type_def import T
 from dataclass_wizard.utils.typing_compat import get_origin, get_args
-from ...conftest import *
 
 
 @pytest.mark.parametrize(
@@ -30,15 +29,8 @@ def test_get_origin(tp, expected):
         (Dict[str, int], (str, int)),
         (int, ()),
         (Callable[[], T][int], ([], int)),
-        # The following cases are an `xfail` on Python 3.6
-        pytest.param(
-            Union[int, Union[T, int], str][int], (int, str),
-                     marks=pytest.mark.skipif(
-                         PY36, reason='requires python 3.7 or higher')),
-        pytest.param(
-            Union[int, Tuple[T, int]][str], (int, Tuple[str, int]),
-            marks=pytest.mark.skipif(
-                PY36, reason='requires python 3.7 or higher')),
+        (Union[int, Union[T, int], str][int], (int, str)),
+        (Union[int, Tuple[T, int]][str], (int, Tuple[str, int])),
     ]
 )
 def test_get_args(tp, expected):
